import re
from urllib.parse import urlparse
from pathlib import Path

from django.apps import apps
from django.conf import settings
from django.db import models
from django.http import HttpResponseBadRequest
from django.urls import reverse

from .mixins import (AlternativeURNResourceMixin, DescriptableResourceMixin,
                     EditableResourceMixin, ValidableResourceMixin)
from .urn import URN


class PassageManager(models.Manager):
    def get_by_natural_key(self, fragment, sub_fragment, *book_natural_key):
        book = apps.get_model("meleager", "Book").objects.get_by_natural_key(*book_natural_key)
        return self.get(fragment=fragment, sub_fragment=sub_fragment, book=book)

    def easy_get(self, *args):
        if not args:
            return None

        if type(args[0]) == str:
            parts = args[0].split(".")
            values = list(parts)
        elif len(args) >= 2:
            values = list(args)
        else:
            raise AttributeError("Invalid attributes")

        if len(values) == 2:
            values.append("")

        book_number, fragment, sub_fragment = int(values[0]), int(values[1]), values[2]
        
        return self.get(book__number=book_number, fragment=fragment, sub_fragment=sub_fragment)


class Passage(
    EditableResourceMixin,
    ValidableResourceMixin,
    DescriptableResourceMixin,
    AlternativeURNResourceMixin,
    models.Model,
):
    """ Passage model """

    objects = PassageManager()

    authors = models.ManyToManyField("meleager.Author")
    cities = models.ManyToManyField(
        "meleager.City",
        related_name="passages",
    )

    book = models.ForeignKey(
        "meleager.Book",
        related_name="passages",
        help_text="Book the passage is in",
        on_delete=models.CASCADE,
        db_index=True,
    )

    fragment = models.IntegerField(
        help_text="The fragment number in book.", db_index=True
    )

    sub_fragment = models.CharField(
        help_text="The sub fragment number in book.",
        blank=True,
        null=False,
        default="",
        max_length=3,
        db_index=True,
    )

    images = models.ManyToManyField(
        "meleager.Medium",
        related_name="passages",
        help_text="The images associated to this resource.",
    )

    comments = models.ManyToManyField(
        "meleager.Comment",
        related_name="passages",
        help_text="The comments of this resource.",
    )

    alternative_uris = models.ManyToManyField("meleager.AlternativeUri")
    external_references = models.ManyToManyField("meleager.ExternalReference", related_name="passages")
    internal_references = models.ManyToManyField("self", symmetrical=True)

    manuscripts = models.ManyToManyField(
        "meleager.Manuscript",
        help_text="The image of the manuscript for this passage.",
        related_name="passages",
    )

    keywords = models.ManyToManyField(
        "meleager.Keyword",
        related_name="passages",
        help_text="The keywords that tag this resource.",
        blank=True,
    )

    def get_absolute_url(self):
        return reverse(
            "web:passage-detail",
            args=(self.book.number, self.fragment, self.sub_fragment),
        )

    @property
    def urn_value(self):
        return Path(urlparse(self.get_absolute_url()).path).name
    
    def natural_key(self):
        return (self.fragment, self.sub_fragment) + self.book.natural_key()
    natural_key.dependencies = ['meleager.Work', 'meleager.Book', 'meleager.Author']

    def __str__(self):
        return f"Passage {self.book.number}.{self.fragment}{self.sub_fragment}"

    class Meta:
        permissions = [
            ("manage_own_passage", "Can manage (edit/delete) their own Passage items")
        ]