from itertools import groupby

from django import forms

from meleager.models import Author, City, Keyword, Passage


class PassageForm(forms.ModelForm):
    fragment = forms.CharField(disabled=True)
    sub_fragment = forms.CharField(disabled=True)

    class Meta:
        model = Passage
        fields = ["fragment", "sub_fragment"]
        exclude = ["urn", "images", "manuscripts", "keywords"]


class AddAuthorToPassageForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        choices = [("", "-------------")]
        choices += sorted(
            [
                (author.pk, ", ".join(name.name for name in author.names.all()))
                for author in Author.objects.all()
            ],
            # Sort by name.
            key=lambda x: x[1],
        )
        self.fields["author"] = forms.ChoiceField(choices=choices)


class AddCityToPassageForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        choices = [("", "-------------")]
        choices += sorted(
            [
                (
                    city.pk,
                    ", ".join(
                        name.name for name in city.names.exclude(cities=None)
                    ),
                )
                for city in City.objects.all()
            ],
            # Sort by name.
            key=lambda x: x[1],
        )
        self.fields["city"] = forms.ChoiceField(choices=choices)


class AddKeywordToPassageForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        choices = [("", "-------------")]
        # We sort choices, first by `category` then by `name` to be able
        # to propose a consistent optgroup on the HTML side.
        keywords = sorted(
            [
                (
                    keyword.pk,
                    ", ".join(name.name for name in keyword.names.all()),
                    keyword.category.names.first().name,
                )
                for keyword in Keyword.objects.select_related("category")
            ],
            # Sort by category.
            key=lambda x: x[2],
        )
        choices += (
            (
                category,
                # Sort by name, only keep pk (0) and name (1).
                sorted([(item[0], item[1]) for item in items], key=lambda x: x[1]),
            )
            for category, items in groupby(keywords, key=lambda x: x[2])
        )
        self.fields["keyword"] = forms.ChoiceField(choices=choices)


class RemoveFromPassageForm(forms.Form):
    pk = forms.CharField(widget=forms.widgets.HiddenInput())
