from django.shortcuts import get_object_or_404, redirect
from django.views import generic

from meleager.models import Language, Passage
from web.forms.comment_description import (
    CommentCreateForm,
    CommentUpdateForm,
    CommentDeleteForm,
)
from web.forms.passage import (
    PassageForm,
    AddAuthorToPassageForm,
    RemoveFromPassageForm,
    AddCityToPassageForm,
    AddKeywordToPassageForm,
)
from web.forms.passage_description import (
    DescriptionCreateForm,
    DescriptionUpdateForm,
    DescriptionDeleteForm,
)
from web.forms.text import AddTextToPassageForm


class PassageDetail(generic.DetailView):
    model = Passage
    template_name = "web/passage/passage.html"
    context_object_name = "passage"

    def get_object(self):
        obj = get_object_or_404(
            Passage,
            book__number=self.kwargs.get("book"),
            fragment=self.kwargs.get("fragment"),
            sub_fragment=self.kwargs.get("sub_fragment", ""),
        )
        return obj

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        passage = self.get_object()
        context["book"] = passage.book
        descriptions = passage.descriptions.all()
        for description in descriptions:
            description.update_form = DescriptionUpdateForm(
                initial={
                    "pk": description.pk,
                    "description": description.description,
                    "language": description.language,
                }
            )
            description.delete_form = DescriptionDeleteForm(
                initial={
                    "pk": description.pk,
                }
            )
        context["descriptions"] = descriptions
        context["description_create_form"] = DescriptionCreateForm()

        comments = passage.comments.all()
        for comment in comments:
            # TODO: handle multiple descriptions?
            description = comment.descriptions.first()
            comment.update_form = CommentUpdateForm(
                initial={
                    "pk": description.pk,
                    "description": description.description,
                    "language": description.language,
                }
            )
            comment.delete_form = CommentDeleteForm(
                initial={
                    "pk": description.pk,
                }
            )
        context["comments"] = comments
        context["comment_create_form"] = CommentCreateForm()

        context["add_text_to_passage_form"] = AddTextToPassageForm()

        def _sort_grc_first(value):
            return value.language.code != "grc"

        context["texts"] = sorted(passage.texts.all(), key=_sort_grc_first)

        authors = passage.authors.all()
        for text in context["texts"]:
            for author in authors:
                for name in author.names.all():
                    if name.language.code == text.language.code:
                        text.author = author
                        text.author_name = name
                author.remove_form = RemoveFromPassageForm(initial={"pk": author.pk})
        context["authors"] = authors
        context["add_author_to_passage_form"] = AddAuthorToPassageForm()

        cities = passage.cities.all()
        for city in cities:
            city.remove_form = RemoveFromPassageForm(initial={"pk": city.pk})
        context["cities"] = cities
        context["add_city_to_passage_form"] = AddCityToPassageForm()

        keywords = passage.keywords.all()
        for keyword in keywords:
            keyword.remove_form = RemoveFromPassageForm(initial={"pk": keyword.pk})
        context["keywords"] = keywords
        context["keywords_categories"] = {keyword.category for keyword in keywords}
        context["add_keyword_to_passage_form"] = AddKeywordToPassageForm()

        context["preferred_languages"] = Language.objects.preferred()
        return context


def passage_detail_pk(request, passage_pk):
    passage = get_object_or_404(Passage, pk=passage_pk)
    return redirect(
        "web:passage-detail",
        passage.book.number,
        passage.fragment,
        passage.sub_fragment,
    )


class PassageUpdate(generic.UpdateView):
    model = Passage
    template_name = "web/passage/passage_form.html"
    form_class = PassageForm
